/*
 *  Copyright (C) 2009 Nicolas Vion <nico@picapo.net>
 *
 *   This file is part of Fluxy.
 *
 *   Fluxy is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Fluxy is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Fluxy; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _AUTODUMP_HH
#define _AUTODUMP_HH

#include <string>
#include "pthread++.hh"
#include "words_parser.hh"
#include "result.hh"
#include "help.hh"

class AutoDumpReport {
private:
	PMutex mutex;
	bool setted;
	time_t at;
	bool result;
	std::string message;
public:
	void set(bool const result, std::string const message);
	void serialize_php(std::stringstream &out);
	void show(std::stringstream &out, std::string const prefix = "");

	AutoDumpReport();
};

class AutoDumpSettings {
private:
	PMutex mutex;

	bool enabled;
	std::string target;
	int delay;

public:
	void set(bool const _enabled, std::string const _target, int const _wait);
	void get(bool &enabled, std::string &target, int &delay);
	void serialize_php(std::stringstream &out);
};

class AutoDump : public PThread {
private:
	PMutex mutex;

protected:
	void main();

public:
	AutoDumpReport report;
	AutoDumpSettings settings;

	bool force();
	bool dump(std::string const target);
	bool parse_query(ClientResult &result, std::string const cmd_prefix, WordsParser *parser, OutputType const mode);
};


#ifdef _AUTODUMP_CC
AutoDump autodump;
#else
extern AutoDump autodump;
#endif

#endif
