/*
 *  Copyright (C) 2009 Nicolas Vion <nico@picapo.net>
 *
 *   This file is part of Fluxy.
 *
 *   Fluxy is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Fluxy is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Fluxy; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "udp.hh"

#include <sys/socket.h>
#include <netdb.h>
#include <sstream>

#include "server.hh"
#include "log.hh"
#include "users.hh"
#include "stringutils.hh"

bool UdpServer::open(std::string const address, std::string const port) {
	log.msg(LOG_NOTICE, "Start udp server on " + address + ":" + port, true);

	addrinfo hints, *res;
	hints.ai_flags = AI_PASSIVE;
	hints.ai_family = AF_INET;
	hints.ai_protocol = IPPROTO_UDP;
	hints.ai_socktype = SOCK_DGRAM;
	hints.ai_addr = NULL;

	//get address/port
	int fodder;
	if ((fodder = getaddrinfo(address.c_str(), port.c_str(), &hints, &res)) != 0) {
		log.msg(LOG_ERR, gai_strerror(fodder), true);
		return false;
	}

	//create socket
	if ((fd = socket(res->ai_family, hints.ai_socktype, hints.ai_protocol)) == -1) {
		freeaddrinfo(res);
		log.msg(LOG_ERR, "Can not create socket", true);
		return false;
	}

	//bind
	if (bind(fd, (struct sockaddr *) res->ai_addr, (socklen_t) res->ai_addrlen) != 0) {
		log.msg(LOG_ERR, "Can not bind socket", true);
		close(fd);
		freeaddrinfo(res);
		return false;
	}

	return true;
}

void UdpServer::exec_query() {
	char buf[1024];
	size_t readen = read(fd, &buf, sizeof(buf));
	if (readen == 0 or readen == sizeof(buf)) {
		return;
	}

	try {
		std::stringstream stream;
		stream.write(buf, readen);
		WordsParser parser(&stream);
		parser.next();

		ClientResult result(NULL);
		users.parse_query(result, &parser, NONE);
	}
	catch (...) {
		log.msg(LOG_ERR, "Not a valid command buffer");
	}
}

void UdpServer::main() {
	for (;;) {
		exec_query();
	}
}
